#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk
# bopscrk - banner and help functions module

from time import sleep
from random import randint

from .color import color
from .transforms import *


# Set the time interval (in secs) between printing each line
interval = 0.03

def banner(name, version, author="r3nt0n"):
    try:
        name_rand_leet = leet_transforms(name)
        name_rand_leet = name_rand_leet[randint(0, (len(name_rand_leet) - 1))]
    except:
        name_rand_leet = name
    name_rand_case = case_transforms(name)
    name_rand_case = name_rand_case[randint((len(name_rand_case) - 3), (len(name_rand_case) - 1))]
    #version = version[:3]
    print('  ,----------------------------------------------------,   ,------------,');sleep(interval)
    print('  | [][][][][]  [][][][][]  [][][][]  [][__]  [][][][] |   |   v{}{}{}   |'.format(color.BLUE, version, color.END));sleep(interval)
    print('  |                                                    |   |------------|');sleep(interval)
    print('  |  [][][][][][][][][][][][][][_]    [][][]  [][][][] |===| {}{}{} |'.format(color.RED, name_rand_leet, color.END));sleep(interval)
    print('  |  [_][][][]{}[]{}[][][][]{}[][]{}[][][ |   [][][]  [][][][] |===| {}{}{}{} |'.format(color.KEY_HIGHL, color.END, color.KEY_HIGHL, color.END, color.BOLD, color.RED, name, color.END));sleep(interval)
    print('  | [][_][]{}[]{}[][][][][]{}[]{}[][][][]||     []    [][][][] |===| {}{}{} |'.format(color.KEY_HIGHL, color.END, color.KEY_HIGHL, color.END, color.RED, name_rand_case, color.END));sleep(interval)
    print('  | [__][][][]{}[]{}[]{}[]{}[][][][][][__]    [][][]  [][][]|| |   |------------|'.format(color.KEY_HIGHL, color.END, color.KEY_HIGHL, color.END));sleep(interval)
    print('  |   [__][________________][__]              [__][]|| |   |{}   {}   {}|'.format(color.GREEN, author, color.END));sleep(interval)
    print('  `----------------------------------------------------´   `------------´\n');sleep(interval)

def help_banner():
    print(u'  +---------------------------------------------------------------------+');sleep(interval)
    print(u'  | Fields can be left empty.  You can use accentuation in your words.  |');sleep(interval)
    print(u'  | If you enable case transforms,  won\'t matter the lower/uppercases   |');sleep(interval)
    print(u'  | in your input. In "others" field (interactive mode), you can write  |');sleep(interval)
    print(u'  | several words comma-separated (e.g.: 2C,Flipper).                   |');sleep(interval)
    print(u'  |                                                                     |');sleep(interval)
    print(u'  |                              For advanced usage and documentation:  |');sleep(interval)
    print(u'  |                                  {}https://github.com/r3nt0n/bopscrk{}  |'.format(color.ORANGE,color.END));sleep(interval)
    print(u'  +---------------------------------------------------------------------+\n');sleep(interval)

def bopscrk_banner():
    sleep(interval * 4)
    print('\n')
    print(u'{}         ▄▄▄▄    ▒█████   ██▓███    ██████  ▄████▄   ██▀███   ██ ▄█▀'.format(color.ORANGE));sleep(interval)
    print(u'        ▓█████▄ ▒██▒  ██▒▓██░  ██▒▒██    ▒ ▒██▀ ▀█  ▓██ ▒ ██▒ ██▄█▒ ');sleep(interval)
    print(u'        ▒██▒ ▄██▒██░  ██▒▓██░ ██▓▒░ ▓██▄   ▒▓█    ▄ ▓██ ░▄█ ▒▓███▄░ ');sleep(interval)
    print(u'        ▒██░█▀  ▒██   ██░▒██▄█▓▒ ▒  ▒   ██▒▒▓▓▄ ▄██▒▒██▀▀█▄  ▓██ █▄ ');sleep(interval)
    print(u'        ░▓█  ▀█▓░ ████▓▒░▒██▒ ░  ░▒██████▒▒▒ ▓███▀ ░░██▓ ▒██▒▒██▒ █▄');sleep(interval)
    print(u'        ░▒▓███▀▒░ ▒░▒░▒░ ▒▓▒░ ░  ░▒ ▒▓▒ ▒ ░░ ░▒ ▒  ░░ ▒▓ ░▒▓░▒ ▒▒ ▓▒');sleep(interval)
    print(u'        ▒░▒   ░   ░ ▒ ▒░ ░▒ ░     ░ ░▒  ░ ░  ░  ▒     ░▒ ░ ▒░░ ░▒ ▒░');sleep(interval)
    print(u'         ░    ░ ░ ░ ░ ▒  ░░       ░  ░  ░  ░          ░░   ░ ░ ░░ ░');sleep(interval)
    print(u'         ░          ░ ░                 ░  ░ ░         ░     ░  ░');sleep(interval)
    print(u'              ░                            ░                        {}'.format(color.END));sleep(interval)
    #sleep(interval*2)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk
# bopscrk - transform functions module

from multiprocessing import Pool, cpu_count
from collections import OrderedDict

from . import Config

def compare(word_to_exclude, word_in_wordlist):
    if word_in_wordlist is not word_to_exclude:
        return word_in_wordlist

# Remove word to exclude from final_wordlist
def multithread_exclude(word_to_exclude, wordlist):
    diff_wordlist = []
    with Pool(cpu_count()) as pool:
        #args = (word, words_to_exclude)
        diff_wordlist += pool.starmap(compare, [(word_to_exclude, word) for word in wordlist])

    # Rewriting here removes excluded words from final_wordlist before it checks next wordlist
    final_wordlist = [word for word in diff_wordlist if word is not None]
    return final_wordlist

def remove_duplicates(wordlist):
    return list(OrderedDict.fromkeys(wordlist))

def remove_by_lengths(wordlist, min_length, max_length):
    '''expect a list, return a new list with the values between min and max length provided'''
    new_wordlist = []
    for word in wordlist:
        #if (len(str(word)) < min_length) or (len(str(word)) > max_length): wordlist.remove(word)
        if (len(str(word)) >= min_length) and (len(str(word)) <= max_length): new_wordlist.append(str(word))
    return new_wordlist


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk

from random import choice

class color:
    PURPLE = u'\033[95m'
    CYAN = u'\033[96m'
    DARKCYAN = u'\033[36m'
    BLUE = u'\033[94m'
    GREEN = u'\033[92m'
    YELLOW = u'\033[93m'
    RED = u'\033[91m'
    BOLD = u'\033[1m'
    UNDERLINE = u'\033[4m'
    ORANGE = u'\033[33m'
    GREY = u'\033[90m'
    #ORANGEBG = '\033[48;2;255;165;0m'
    END = u'\033[0m'

    RAND_KEY_COLOR = [PURPLE, CYAN, DARKCYAN, YELLOW, ORANGE]
    KEY_HIGHL = choice(RAND_KEY_COLOR)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk
# bopscrk - parsing configuration module

import configparser

#CFG_FILE_example = './bopscrk.cfg'  # path relative to bopscrk.py

class Config:
    def __init__(self, cfg_file):
        self.CFG_FILE = cfg_file
        self.cfg = configparser.ConfigParser(strict=False)

    def read_config(self, category, field):
        try:
            self.cfg.read([self.CFG_FILE])
            value = self.cfg.get(category, field)
        except Exception as e:
            print(e)
            value = False
        return value

    def merge_settings(self, chars, strings):
        final_list = []
        if chars:
            final_list[:] = chars
        if strings:
            final_list.extend(strings.split())
        return final_list

    def parse_booleans(self, value):
        try:
            if value.lower() == 'true':
                return True
            return False
        except AttributeError:
            return None

    def setup(self):
        self.EXTRA_COMBINATIONS = self.parse_booleans(self.read_config('COMBINATIONS', 'extra_combinations'))
        self.SEPARATORS_CHARSET = self.merge_settings(self.read_config('COMBINATIONS', 'separators_chars'),
                                                      self.read_config('COMBINATIONS', 'separators_strings'))
        self.LEET_CHARSET = (self.read_config('TRANSFORMS', 'leet_charset')).split()
        self.RECURSIVE_LEET = self.parse_booleans(self.read_config('TRANSFORMS', 'recursive_leet'))
        self.REMOVE_PARENTHESIS = self.parse_booleans(self.read_config('LYRICS', 'remove_parenthesis'))
        self.TAKE_INITIALS = self.parse_booleans(self.read_config('LYRICS', 'take_initials'))
        self.ARTIST_SPLIT_BY_WORD = self.parse_booleans(self.read_config('LYRICS', 'artist_split_by_word'))
        self.LYRIC_SPLIT_BY_WORD = self.parse_booleans(self.read_config('LYRICS', 'lyric_split_by_word'))
        self.ARTIST_SPACE_REPLACEMENT = self.parse_booleans(self.read_config('LYRICS', 'artist_space_replacement'))
        self.LYRIC_SPACE_REPLACEMENT = self.parse_booleans(self.read_config('LYRICS', 'lyric_space_replacement'))
        self.SPACE_REPLACEMENT_CHARSET = self.merge_settings(self.read_config('LYRICS', 'space_replacement_chars'),
                                                             self.read_config('LYRICS', 'space_replacement_strings'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk
# bopscrk - transform functions module

from multiprocessing import cpu_count, Pool

from alive_progress import alive_bar

from . import Config
from .excluders import remove_duplicates
from .auxiliars import append_wordlist_to_file


def case_transforms(word):
    new_wordlist = []

    # Make each one upper (hello => Hello, hEllo, heLlo, helLo, hellO)
    i=0
    for char in word:
        new_word = word[:i] + char.upper() + word[i+1:]
        i += 1
        if new_word not in new_wordlist: new_wordlist.append(new_word)

    # Make pairs upper (hello => HeLlO)
    i=0
    new_word = ''
    for char in word:
        if i % 2 == 0: new_word += char.upper()
        else: new_word += char
        i += 1
    if new_word not in new_wordlist: new_wordlist.append(new_word)

    # Make odds upper (hello => hElLo)
    i=0
    new_word = ''
    for char in word:
        if i % 2 != 0: new_word += char.upper()
        else: new_word += char
        i += 1
    if new_word not in new_wordlist: new_wordlist.append(new_word)

    # Make consonants upper (hello => HeLLo)
    vowels = 'aeiou'
    new_word = ''
    for char in word:
        if char.lower() not in vowels: new_word += char.upper()
        else: new_word += char
    if new_word not in new_wordlist: new_wordlist.append(new_word)

    # Make vowels upper (hello => hEllO)
    new_word = ''
    for char in word:
        if char.lower() in vowels: new_word += char.upper()
        else: new_word += char
    if new_word not in new_wordlist: new_wordlist.append(new_word)

    # recursive call function (not working, maybe this option won't be even useful)
    # for new_word in new_wordlist:
    #     original_size = len(new_wordlist)
    #     new_wordlist.extend(case_transforms(new_word))
    #     if len(new_wordlist) == original_size:
    #         break  # breaking recursive call

    return new_wordlist


def leet_transforms(word):
    new_wordlist = []
    original_size = len(new_wordlist)
    i=0
    leet_charset = Config.LEET_CHARSET
    for char in word:
        for lchar in leet_charset:
            leeted_char = ''
            if lchar.startswith(char.lower()):
                leeted_char = lchar[-1:]
                new_word = word[:i] + leeted_char + word[i + 1:]
                if new_word not in new_wordlist: new_wordlist.append(new_word)
                # dont break to allow multiple transforms to a single char (e.g. a into 4 and @)
        i += 1

    # MULTITHREAD RECURSIVE call function (doesn't seem efficient)
    # if Config.RECURSIVE_LEET and (len(new_wordlist) > original_size):
    #     new_wordlist += multithread_transforms(leet_transforms, new_wordlist)

    # UNITHREAD RECURSIVE call function
    if Config.RECURSIVE_LEET:
        for new_word in new_wordlist:
            original_size = len(new_wordlist)
            new_wordlist.extend(leet_transforms(new_word))
            if len(new_wordlist) == original_size:
                break  # breaking recursive call

    return remove_duplicates(new_wordlist)


def take_initials(word):
    splitted = word.split(' ')
    initials = ''
    for char in splitted:
        try: initials += char[0]
        except IndexError: continue
    return initials


def artist_space_transforms(word):
    new_wordlist = []
    if ' ' in word:  # Avoiding non-space words to be included many
        if Config.ARTIST_SPLIT_BY_WORD:
            # Add each word in the artist name splitting by spaces (e.g.: ['bob', 'dylan'])
            new_wordlist.extend(word.split(' '))
        # Add artist name without spaces (e.g.: 'bobdylan')
        new_wordlist.append(word.replace(' ', ''))
        # Replace spaces in artist name with all space replacements charset
        if (Config.ARTIST_SPACE_REPLACEMENT and Config.SPACE_REPLACEMENT_CHARSET):
            for character in Config.SPACE_REPLACEMENT_CHARSET:
                new_wordlist.append(word.replace(' ', character))

    return new_wordlist


def lyric_space_transforms(word):
    new_wordlist = []
    if ' ' in word:  # Avoiding non-space words to be included many
        if Config.LYRIC_SPLIT_BY_WORD:
            # Add each word in the phrase splitting by spaces (e.g.: ['hello', 'world'])
            new_wordlist.extend(word.split(' '))
        # Add phrase without spaces (e.g.: 'helloworld')
        new_wordlist.append(word.replace(' ', ''))
        # Replace spaces in phrase with all space replacements charset
        if (Config.LYRIC_SPACE_REPLACEMENT and Config.SPACE_REPLACEMENT_CHARSET):
            for character in Config.SPACE_REPLACEMENT_CHARSET:
                new_wordlist.append(word.replace(' ', character))
    return new_wordlist


def multiprocess_transforms(transform_type, wordlist):
    # process each word in their own thread and return the results
    new_wordlists = []
    with Pool(cpu_count()) as pool:
        with alive_bar(bar=None,spinner='bubbles', monitor=False,elapsed=False,stats=False,receipt=False) as progressbar:
            new_wordlists += pool.map(transform_type, wordlist)
            progressbar()
    new_wordlist = []
    for nlist in new_wordlists:
         new_wordlist += nlist
    return new_wordlist


def transform_cached_wordlist_and_save(transform_type, filepath):

    last_position = 0

    while True:

        cached_wordlist = []
        new_wordlist = []

        with open(filepath, 'r', encoding='utf-8') as f:
            counter = 0
            f.seek(last_position)  # put point on last position
            while True:
                line = f.readline()
                if counter >= 8000:
                    last_position = f.tell()  # save last_position and break inner loop
                    break
                if not line:
                    break
                if line.strip() not in cached_wordlist:
                    cached_wordlist.append(line.strip())
                counter += 1
                last_position = f.tell()  # save last_position

        new_wordlist += multiprocess_transforms(transform_type, cached_wordlist)
        #cached_wordlist += new_wordlist
        append_wordlist_to_file(filepath, new_wordlist)

        # read again the file to check if it ended
        with open(filepath, 'r', encoding='utf-8') as f:
            f.seek(last_position)  # put point on last position
            line = f.readline()
            if not line:
                break






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk


from .args import Arguments
from .config import Config


args = Arguments()
Config = Config(args.cfg_file)
#Config.setup()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk
# bopscrk - init script


name = 'bopscrk.py'
desc = 'Generate smart and powerful wordlists'
__version__ = '2.4.6'
__author__ = 'r3nt0n'
__status__ = 'Development'


def start():
    try:
        from .modules import main
    except ImportError:
        # catching except when running python3 bopscrk.py
        # (sketchy, need some refactor)
        from modules import main

    main.run(name, __version__)


if __name__ == '__main__':
    start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# r3nt0n

import os, sys, argparse

from .color import color
from .auxiliars import is_empty, is_valid_date


class Arguments:
    def __init__(self):
        self.DEFAULT_MIN = 4
        self.DEFAULT_MAX = 12
        self.DEFAULT_N_WORDS = 2
        self.DEFAULT_OUTPUT_FILE = 'tmp.txt'
        # scratch, still need to be regorganized
        import os
        self.DEFAULT_CFG_FILE = os.path.abspath(
                os.path.join(os.path.dirname(os.path.abspath(__file__)),
                             '../bopscrk.cfg')
        )
        #self.DEFAULT_CFG_FILE = './bopscrk.cfg'

        parser = argparse.ArgumentParser(description='Generates smart and powerful wordlists.')

        parser.add_argument('-i', '--interactive', action="store_true",
                            help='interactive mode, the script will ask you about target')

        parser.add_argument('-w', action="store", metavar='', type=str, dest='words',
                            help='words to combine comma-separated (will be combined with all words)')

        parser.add_argument('-m', '--min', action="store", metavar='', type=int, dest='min',
                            default=self.DEFAULT_MIN, help='min length for the words to generate (default: {})'.format(self.DEFAULT_MIN))

        parser.add_argument('-M', '--max', action="store", metavar='', type=int, dest='max',
                            default=self.DEFAULT_MAX, help='max length for the words to generate (default: {})'.format(self.DEFAULT_MAX))

        parser.add_argument('-c', '--case', action="store_true", help='enable case transformations')

        parser.add_argument('-l', '--leet', action="store_true", help='enable leet transformations')

        parser.add_argument('-n', action="store", metavar='', type=int, dest='n_words',
                            default=self.DEFAULT_N_WORDS, help='max amount of words to combine each time '
                                            '(default: {})'.format(self.DEFAULT_N_WORDS))
        parser.add_argument('-a', '--artists', action="store", metavar='', type=str,
                            dest='artists', default=False,
                            help='artists to search song lyrics (comma-separated)')

        # parser.add_argument('-x', '--exclude', action="store", metavar='', type=str,
        #                     dest='exclude', default=False,
        #                     help='exclude all the words included in other wordlists '
        #                          '(several wordlists should be comma-separated)')

        parser.add_argument('-o', '--output', action="store", metavar='', type=str,
                            dest='outfile', default=self.DEFAULT_OUTPUT_FILE,
                            help='output file to save the wordlist (default: {})'.format(self.DEFAULT_OUTPUT_FILE))

        parser.add_argument('-C', '--config', action="store", metavar='', type=str,
                            dest='cfg_file', default=self.DEFAULT_CFG_FILE,
                            help='specify config file to use (default: {})'.format(self.DEFAULT_CFG_FILE))

        parser.add_argument('--version', action="store_true", help='print version and exit')

        self.parser = parser
        self.args = parser.parse_args()
        self.interactive = self.args.interactive
        self.cfg_file = self.args.cfg_file
        self.print_version = self.args.version

    def set_interactive_options(self):
        while True:
            min_length = input('  {}[?]{} Passwords min length [{}] >>> '.format(color.BLUE, color.END, self.DEFAULT_MIN))
            if is_empty(min_length):
                self.min_length = self.DEFAULT_MIN; break
            else:
                try:
                    self.min_length = int(min_length)
                    break
                except ValueError:
                    print('  {}[!]{} Min length should be an integer'.format(color.RED, color.END))
        while True:
            max_length = input('  {}[?]{} Password\'s max length [{}] >>> '.format(color.BLUE, color.END, self.DEFAULT_MAX))
            if is_empty(max_length):
                self.max_length = self.DEFAULT_MAX; break
            else:
                try:
                    max_length = int(max_length)
                    if max_length < self.min_length:
                        print('  {}[!]{} Max should be greater or equal than min'.format(color.RED, color.END))
                    else:
                        self.max_length = max_length
                        break
                except ValueError:
                    print('  {}[!]{} Max length should be an integer'.format(color.RED, color.END))

        firstname = input('  {}[?]{} First name >>> '.format(color.BLUE, color.END))
        surname = input('  {}[?]{} Surname >>> '.format(color.BLUE, color.END))
        lastname = input('  {}[?]{} Last name >>> '.format(color.BLUE, color.END))

        while True:
            birth = input('  {}[?]{} Birth date (DD/MM/YYYY) >>> '.format(color.BLUE, color.END))
            if not is_empty(birth) and not is_valid_date(birth):
                print('  {}[!]{} Birthdate wrong format'.format(color.RED, color.END))
            else:
                break

        self.artists = input('  {}[?]{} Artist names to search song lyrics (comma-separated) >>> '.format(color.BLUE, color.END))
        if is_empty(self.artists):
            self.artists = False
        else:
            self.artists = self.artists.split(',')

        others = input('  {}[?]{} Some other relevant words (comma-separated) >>> '.format(color.BLUE, color.END))

        leet = input('  {}[?]{} Do yo want to make leet transforms? [y/n] >>> '.format(color.BLUE, color.END))
        case = input('  {}[?]{} Do yo want to make case transforms? [y/n] >>> '.format(color.BLUE, color.END))

        if leet.lower() == 'y': self.leet = True
        else: self.leet = False

        if case.lower() == 'y': self.case = True
        else: self.case = False

        while True:
            n_words = input('  {}[?]{} How much words do you want to combine at most [{}] >>> '.format(color.BLUE, color.END, self.DEFAULT_N_WORDS))
            if is_empty(n_words):
                self.n_words = self.DEFAULT_N_WORDS; break
            else:
                try:
                    n_words = int(n_words)
                    if n_words < 1:
                        print('  {}[!]{} Should be greater or equal than 1'.format(color.RED, color.END))
                    else:
                        self.n_words = n_words
                        break
                except ValueError:
                    print('  {}[!]{} Should be an integer'.format(color.RED, color.END))

        # while True:
        #     exclude = input('  {}[?]{} Exclude words from other wordlists? >>> '.format(color.BLUE, color.END))
        #     if is_empty(exclude):
        #         self.exclude_wordlists = False; break
        #     else:
        #         exclude = exclude.split(',')
        #         valid_paths = True
        #         for wl_path in exclude:
        #             if not os.path.isfile(wl_path):
        #                 valid_paths = False
        #                 print('  {}[!]{} {} not found'.format(color.RED, color.END, wl_path))
        #         if valid_paths:
        #             self.exclude_wordlists = exclude
        #             break

        self.outfile = input('  {}[?]{} Output file [{}] >>> '.format(color.BLUE, color.END, self.DEFAULT_OUTPUT_FILE))
        if is_empty(self.outfile): self.outfile = self.DEFAULT_OUTPUT_FILE

        print('')  # Print blank line after all questions

        self.base_wordlist = []
        # here I can select on which wordlist include each info by their weight (to implement)
        if not is_empty(firstname):
            firstname = firstname.lower()
            self.base_wordlist.append(firstname)
        if not is_empty(surname):
            surname = surname.lower()
            self.base_wordlist.append(surname)
        if not is_empty(lastname):
            lastname = lastname.lower()
            self.base_wordlist.append(lastname)
        if not is_empty(birth):
            birth = birth.split('/')
            for i in birth:
                self.base_wordlist.append(i)
            self.base_wordlist.append((birth[2])[-2:])  # Also add two last digits of the year
        if not is_empty(others):
            others = others.split(',')
            for i in others:
                self.base_wordlist.append(i.lower())

    def set_cli_options(self):
        self.base_wordlist = []
        if self.args.words:
            [self.base_wordlist.append(word.lower()) for word in ((self.args.words).split(','))]
        self.min_length = self.args.min
        self.max_length = self.args.max
        self.leet = self.args.leet
        self.case = self.args.case
        self.n_words = self.args.n_words
        self.artists = self.args.artists
        self.outfile = self.args.outfile
        # self.exclude_wordlists = self.args.exclude
        # if self.exclude_wordlists:
        #     self.exclude_wordlists = self.exclude_wordlists.split(',')
        #     for wl_path in self.exclude_wordlists:
        #         if not os.path.isfile(wl_path):
        #             print('  {}[!]{} {} not found'.format(color.RED, color.END, wl_path))
        #             sys.exit(4)
        if self.artists:
            self.artists = self.artists.split(',')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk
# bopscrk - script opts and presentation

import sys, os, datetime

#from bopscrk.bopscrk import name, __version__, __author__
from . import args, Config
from .auxiliars import clear, remove_duplicates_from_file
from . import banners
from .color import color
from .transforms import leet_transforms, case_transforms, artist_space_transforms, lyric_space_transforms, multiprocess_transforms, take_initials, transform_cached_wordlist_and_save
from .combinators import combinator, add_common_separators
from .excluders import remove_by_lengths, remove_duplicates, multithread_exclude


def run(name, version):
    # check Python version
    if sys.version_info < (3, 0): print('Python 3 is required'); sys.exit(1)
    # Print simple help and exit when runs without args
    if len(sys.argv) == 1: args.parser.print_help(sys.stdout); sys.exit(2)
    # Print version and exit (when runs with -v)
    if args.print_version: print(name + '_' + version); sys.exit(0)

    try:
        # setting args whether interactive or not
        if args.interactive:
            clear()
            banners.bopscrk_banner()
            banners.help_banner()
            banners.banner(name, version)
            args.set_interactive_options()
        else:
            banners.bopscrk_banner()
            banners.help_banner()
            banners.banner(name, version)
            args.set_cli_options()

        # Check if config file exists
        if not os.path.exists(args.cfg_file):
            print('  {}[!]{} error trying to load config file {}'.format(color.RED, color.END, args.cfg_file))
            sys.exit(3)

        else:
            Config.setup()
            print('  {}[V]{} config file {} loaded'.format(color.GREEN, color.END, args.cfg_file))

        # Initial timestamp
        start_time = datetime.datetime.now().time().strftime('%H:%M:%S')

        # Inserting original values into final_wordlist
        base_wordlist = args.base_wordlist
        print('  {}[+]{} Appending words provided (base wordlist length: {})...'.format(color.BLUE, color.END, len(base_wordlist)))
        final_wordlist = base_wordlist[:]  # Copy to preserve the original

        # SEARCH FOR LYRICS
        if args.artists:
            print('  {}[+]{} Appending artist names   (base wordlist length: {})...'.format(color.BLUE, color.END,(len(base_wordlist)+len(args.artists))))
            for artist in args.artists:
                # Add IN BASE WORDLIST artist name as a word
                base_wordlist.append(artist)

                # Add artist name with all space transformed configured (and enabled) into a specific charset
                if not (Config.SPACE_REPLACEMENT_CHARSET and Config.ARTIST_SPACE_REPLACEMENT):
                    print('  {}[!]{} Any space-replacement charset specified in {}'.format(color.ORANGE, color.END, args.cfg_file))
                    print('  {}[!]{} Spaces inside artists names won\'t be replaced\n'.format(color.ORANGE, color.END))
                elif Config.ARTIST_SPACE_REPLACEMENT:
                    print('  {}[+]{} Producing new words replacing any space in {}...'.format(color.BLUE, color.END,artist))
                    final_wordlist += artist_space_transforms(artist)

                # Search lyrics if it meets dependencies for lyricpass
                try:
                    from .lyricpass import lyricpass
                    print('\n{}     -- Starting lyricpass module --\n'.format(color.GREY))
                    print('  {}[*]{} Looking for {}\'s lyrics...'.format(color.CYAN, color.END, artist.title()))
                    lyrics = lyricpass.lyricpass(artist)
                    #lyrics = [s.decode("utf-8") for s in lyfinder.lyrics]
                    print('\n  {}[*] {}{}{} phrases found'.format(color.CYAN, color.GREEN, len(lyrics), color.END))
                    print('\n{}     -- Stopping lyricpass module --\n'.format(color.GREY))

                    # First we remove all the parenthesis in the phrases (if enabled)
                    if Config.REMOVE_PARENTHESIS:
                        lyrics = ([s.replace('(', '') for s in lyrics])
                        lyrics = ([s.replace(')', '') for s in lyrics])

                    # Add the phrases to BASE wordlist
                    lyrics = remove_by_lengths(lyrics, args.min_length, args.max_length)
                    print('  {}[+]{} Adding raw phrases filtering by min and max length range ({} phrases remain)...'.format(color.BLUE, color.END,len(lyrics)))
                    final_wordlist += lyrics

                    # Take just the initials on each phrase and add as a new word to FINAL wordlist
                    if Config.TAKE_INITIALS:
                        base_lyrics = lyrics[:]
                        ly_initials_wordlist = multiprocess_transforms(take_initials, base_lyrics)
                        final_wordlist += ly_initials_wordlist

                    # Make space transforms and add it too
                    if not (Config.SPACE_REPLACEMENT_CHARSET and Config.LYRIC_SPACE_REPLACEMENT):
                        print('  {}[!]{} Any spaces-replacement charset specified in {}'.format(color.ORANGE, color.END, args.cfg_file))
                        print('  {}[!]{} Spaces inside lyrics won\'t be replaced\n'.format(color.ORANGE,color.END))
                    elif Config.LYRIC_SPACE_REPLACEMENT:
                        print('  {}[+]{} Producing new words replacing spaces in {} phrases...'.format(color.BLUE, color.END, len(lyrics)))
                        base_lyrics = lyrics[:]
                        space_transformed_lyrics = multiprocess_transforms(lyric_space_transforms, base_lyrics)
                        final_wordlist += space_transformed_lyrics

                except ImportError:
                    print('  {}[!]{} missing dependencies, only artist names will be added and transformed'.format(color.RED, color.END))

        # WORD COMBINATIONS
        if ((args.n_words > 1)):
            print('  {}[+]{} Creating all posible combinations between words...'.format(color.BLUE, color.END))
            i = 1
            while ((i < args.n_words) and (len(base_wordlist) > i)):
                i += 1
                final_wordlist += combinator(base_wordlist, i)
                print('  {}[*]{} {} words combined using {} words (words produced: {})'.format(color.CYAN,color.END,len(base_wordlist),i, len(final_wordlist)))

        # WORD COMBINATIONS (WITH COMMON SEPARATORS)
        if Config.EXTRA_COMBINATIONS:
            if Config.SEPARATORS_CHARSET:
                #print('  {}[+]{} Creating extra combinations (separators charset in {}{}{})...'.format(color.BLUE, color.END,color.CYAN, args.cfg_file,color.END))
                print('  {}[+]{} Creating extra combinations using separators charset...'.format(color.BLUE,color.END))
                final_wordlist += add_common_separators(base_wordlist)
                print('  {}[*]{} Words produced: {}'.format(color.CYAN, color.END, len(final_wordlist)))
            else:
                print('  {}[!]{} No separators charset specified in {}{}'.format(color.ORANGE, color.END, args.cfg_file,color.END))

        # Remove words by min-max length range established
        print('  {}[-]{} Removing words by min and max length provided ({}-{})...'.format(color.PURPLE, color.END,args.min_length,args.max_length))
        final_wordlist = remove_by_lengths(final_wordlist, args.min_length, args.max_length)
        print('  {}[*]{} Words remaining: {}'.format(color.CYAN, color.END, len(final_wordlist)))
        # (!) Check for duplicates (is checked before return in combinator() and add_common_separators())
        #final_wordlist = remove_duplicates(final_wordlist)


        # # CASE TRANSFORMS
        # if args.case:
        #     print('  {}[+]{} Applying case transforms to {} words...'.format(color.BLUE, color.END, len(final_wordlist)))
        #
        #     # transform_cached_wordlist_and_save(case_transforms, args.outfile) # not working yet, infinite loop ?¿?¿
        #     temp_wordlist = []
        #     temp_wordlist += multithread_transforms(case_transforms, final_wordlist)
        #     final_wordlist += temp_wordlist
        #
        # final_wordlist = remove_duplicates(final_wordlist)
        #
        # # SAVE WORDLIST TO FILE BEFORE LEET TRANSFORMS
        # ############################################################################
        # with open(args.outfile, 'w') as f:
        #     for word in final_wordlist:
        #         f.write(word + '\n')

        # LEET TRANSFORMS
        if args.leet:
            if not Config.LEET_CHARSET:
                print('  {}[!]{} Any leet charset specified in {}'.format(color.ORANGE, color.END, args.cfg_file))
                print('  {}[!]{} Skipping leet transforms...'.format(color.ORANGE, color.END, args.cfg_file))
            else:
                recursive_msg = ''
                if Config.RECURSIVE_LEET:
                    # print('\n  {}[!] WARNING: Recursive leet is enabled, depending on the words\n'
                    #       '      max-length configured (now is {}{}{}) and the size of your\n'
                    #       '      wordlist at this point (now contains {}{}{} words), this process\n'
                    #       '      could take a long time{}\n'.format(color.ORANGE,color.END,args.max_length,color.ORANGE,color.END,len(final_wordlist),color.ORANGE,color.END))
                    recursive_msg = '{}recursive{} '.format(color.ORANGE,color.END)
                print('  {}[+]{} Applying {}leet transforms to {} words...'.format(color.BLUE, color.END, recursive_msg,len(final_wordlist)))

                #transform_cached_wordlist_and_save(leet_transforms, args.outfile)
                #remove_duplicates_from_file(args.outfile)

                temp_wordlist = []
                temp_wordlist += multiprocess_transforms(leet_transforms, final_wordlist)
                final_wordlist += temp_wordlist

        # CASE TRANSFORMS
        if args.case:
            print('  {}[+]{} Applying case transforms to {} words...'.format(color.BLUE, color.END,len(final_wordlist)))

            # transform_cached_wordlist_and_save(case_transforms, args.outfile) # not working yet, infinite loop ?¿?¿

            temp_wordlist = []
            temp_wordlist += multiprocess_transforms(case_transforms, final_wordlist)
            final_wordlist += temp_wordlist

        print('  {}[-]{} Removing duplicates...'.format(color.PURPLE, color.END))
        final_wordlist = remove_duplicates(final_wordlist)
        print('  {}[*]{} Words remaining: {}'.format(color.CYAN, color.END, len(final_wordlist)))

        # EXCLUDE FROM OTHER WORDLISTS (deprecated)
        #if args.exclude_wordlists:
            # For each path to wordlist provided
            # for wl_path in args.exclude_wordlists:
            #     print('  {}[+]{} Excluding wordlist {}...'.format(color.BLUE, color.END, os.path.basename(wl_path)))
            #     # Open the file
            #     with open(wl_path, 'r') as x_wordlist_file:
            #         # Read line by line in a loop
            #         while True:
            #             word_to_exclude = x_wordlist_file.readline()
            #             if not word_to_exclude: break  # breaks the loop when file ends
            #             final_wordlist = multithread_exclude(word_to_exclude, final_wordlist)

        # re-check for duplicates
        #final_wordlist = remove_duplicates(final_wordlist)

        # SAVE WORDLIST TO FILE
        ###########################################################################
        with open(args.outfile, 'w') as f:
            for word in final_wordlist:
                f.write(word + '\n')

        # Final timestamps
        end_time = datetime.datetime.now().time().strftime('%H:%M:%S')
        total_time = (datetime.datetime.strptime(end_time, '%H:%M:%S') -
                      datetime.datetime.strptime(start_time, '%H:%M:%S'))

        # PRINT RESULTS
        ############################################################################
        print('\n  {}[+]{} Words generated:\t{}{}{}'.format(color.GREEN, color.END, color.RED, len(final_wordlist),color.END))
        print('  {}[+]{} Elapsed time:\t{}'.format(color.GREEN, color.END, total_time))
        print('  {}[+]{} Output file:\t{}{}{}{}'.format(color.GREEN, color.END, color.BOLD, color.BLUE, args.outfile, color.END))
        #print('  {}[+]{} Words generated:\t{}{}{}\n'.format(color.GREEN, color.END, color.RED, str(sum(1 for line in open(args.outfile))), color.END))
        sys.exit(0)

    except KeyboardInterrupt:
        print('\n\n  {}[!]{} Exiting...\n'.format(color.RED, color.END))
        sys.exit(3)



#!/usr/bin/env python3
"""
Utility to scrape lyrics from https://lyrics.com

Usage:
lyricspass.py -a <artist>
lyricpass.py -i <file with multiple artists>

Example:
python lyricpass.py -a "Rob Zombie"
python lyricpass.py -i /tmp/artists.txt

Outputs two files:
raw-lyrics.txt <everything>
wordlist.txt <cleaned passphrases>

Tool by initstring. If you're into cracking complex passwords, check out
github.com/initstring/passphrase-wordlist for more fun!

This is a modified version by r3nt0n integrated into bopscrk:
https://github.com/r3nt0n/bopscrk
"""

#import argparse
import urllib.request
#import datetime
#import os
#import sys
import re

SITE = "https://www.lyrics.com/"
# LYRIC_FILE = "raw-lyrics-{:%Y-%m-%d-%H.%M.%S}".format(datetime.datetime.now())  # r3nt0n: i don't need this file
# PASS_FILE = "wordlist-{:%Y-%m-%d-%H.%M.%S}".format(datetime.datetime.now())  # r3nt0n: i don't need this file
HEADER = {"User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/119.0"}
BOPSCRK_INDENT = "  "

# r3nt0n: i don't need this function
# def parse_args():
#     """
#     Handle user-passed parameters
#     """
#     desc = "Scrape song lyrics from wikia.com"
#     parser = argparse.ArgumentParser(description=desc)
#
#     group = parser.add_mutually_exclusive_group(required=True)
#     group.add_argument("-a", "--artist", type=str, action="store",
#                        help="Single artist to scrape")
#     group.add_argument("-i", "--infile", type=str, action="store",
#                        help="File containing one artist per line to scrape")
#
#     parser.add_argument("--min", type=int, default=8,
#                         help="Minimum passphrase length. Default=8")
#     parser.add_argument("--max", type=int, default=40,
#                         help="Minimum passphrase length. Default=40")
#
#     args = parser.parse_args()
#
#     if args.infile:
#         if not os.access(args.infile, os.R_OK):
#             print("[!] Cannot access input file, exiting")
#             sys.exit()
#
#     return args

def make_phrases(line):
    """
    Cleans raw lyrics into usable passphrases
    """
    clean_lines = []
    final_lines = []

    # Allow only letters, numbers, spaces, and some punctuation
    allowed_chars = re.compile("[^a-zA-Z0-9 '&]")

    # Lowercase everything and deal with common punctuation
    line = line.lower()
    line = re.sub(r'[-_]', ' ', line)

    # The following lines attempt to remove accented characters, as the
    # tool is focused on Engligh-language passwords.
    line = re.sub('[àáâãäå]', 'a', line)
    line = re.sub('[èéêë]', 'e', line)
    line = re.sub('[ìíîï]', 'i', line)
    line = re.sub('[òóôõö]', 'o', line)
    line = re.sub('[ùúûü]', 'u', line)
    line = re.sub('[ñ]', 'n', line)

    # Gets rid of any remaining special characters in the name
    line = allowed_chars.sub('', line)

    # Shrinks down multiple spaces
    line = re.sub(r'\s\s+', ' ', line)

    # If line has an apostrophe make a duplicate without
    if "'" in line:
        clean_lines.append(re.sub("'", "", line))

    # Making duplicating phrases including and / &
    if ' and ' in line:
        clean_lines.append(re.sub(' and ', ' & ', line))
    if '&' in line:
        newline = re.sub('&', ' and ', line)
        newline = re.sub(r'\s+', ' ', newline).strip()
        clean_lines.append(newline)

    # Add what is left to the list
    clean_lines.append(line)

    # Only keep items in the acceptable length
    # for item in clean_lines:
    #     if args.max >= len(item) >= args.min:
    #         final_lines.append(item)
    final_lines = clean_lines

    return final_lines

def parse_artists(bopscrk_artist):
    """
    Return a list of song artists for parsing
    r3nt0n: bopscrk always provides a list as arg, doesn't need to split or read the file here
    """
    whitelist = re.compile('[^a-zA-Z0-9-+]')
    artists = []

    # if args.artist:
    raw_artists = [bopscrk_artist,]
    # else:
    #     with open(args.infile, encoding="utf-8", errors="ignore") as infile:
    #         raw_artists = infile.readlines()

    for artist in raw_artists:
        artist = artist.replace(" ", "+")
        artist = whitelist.sub("", artist)
        if artist not in artists:
            artists.append(artist)

    return artists

def build_urls(artist):
    """
    Creates a list of song URLs for a specific artist
    """
    not_found = "We couldn't find any artists matching your query"
    query_url = SITE + "/artist.php?name=" + artist
    song_ids = []
    regex = re.compile(r'href="/lyric/(.*?)/')

    req = urllib.request.Request(query_url, headers=HEADER)
    with urllib.request.urlopen(req) as response:
        html = response.read().decode()

    # The songs are stored by a unique ID
    song_ids = re.findall(regex, html)

    if not_found in html:
        print("{}[!] Artist {} not found, skipping".format(BOPSCRK_INDENT, artist))

        # Clear out the "suggested" songs it finds in this scenario
        song_ids = []
    elif not song_ids:
        print("{}[!] No songs found for {}, skipping".format(BOPSCRK_INDENT,artist))
    else:
        print("{}[+] Found {} songs for artists {}".format(BOPSCRK_INDENT,len(song_ids), artist))

    # The "print" URL shows us the easiest to decode version of the song
    url_list = [SITE + "db-print.php?id=" + id for id in song_ids]

    return url_list

# def write_data(outfile, data):
#     """
#     Generic helper function to write text to a file
#     """
#     with open(outfile, "a") as open_file:
#         for line in data:
#             if line:
#                 open_file.write(line + '\n')

def scrape_lyrics(url_list):
    """
    Scrapes raw lyric data from a list of URLs
    """
    regex = re.compile(r"<pre.*?>(.*?)</pre>", re.DOTALL)
    newline = re.compile(r"\r\n|\n")

    deduped_lyrics = set()

    current = 1
    total = len(url_list)

    for url in url_list:
        print("{}[+] Checking song {}/{}...       \r".format(BOPSCRK_INDENT,current, total), end="")

        req = urllib.request.Request(url, headers=HEADER)
        with urllib.request.urlopen(req) as response:
            html = response.read().decode()

        lyrics = re.findall(regex, html)

        # We should always have a match... but if not, skip this url
        if not lyrics:
            print("\n{}[!] Found no lyrics at {}".format(BOPSCRK_INDENT,url))
            continue

        lyrics = re.split(newline, lyrics[0])

        #write_data(LYRIC_FILE, lyrics)  # r3nt0n: i don't need this file

        deduped_lyrics.update(lyrics)

        current += 1

    return deduped_lyrics


#def main():
def lyricpass(bopscrk_artist):
    """
    Main program function
    """
    #args = parse_args()
    artists = parse_artists(bopscrk_artist)

    raw_words = set()
    final_phrases = set()

    # First, we grab all the lyrics for a given artist.
    # The scrape_lyrics function will write the raw lyrics to an output
    # file as it goes, which may come in handy if the program exits early
    # due to an error.
    for artist in artists:
        #print("[+] Looking up artist {}".format(artist))  # r3nt0n: shutting up prints
        url_list = build_urls(artist)
        if not url_list:
            continue
        raw_words.update(scrape_lyrics(url_list))

    # Now we will apply some rules to clean all the raw lyrics into a base
    # passphrase file that can be used for cracking.
    for lyric in raw_words:
        phrases = make_phrases(lyric)
        final_phrases.update(phrases)

    return final_phrases

    # Write out the cleaned passphrases to a file
    # write_data(PASS_FILE, final_phrases)

    # print("[+] All done!")
    # print("")
    # print("Raw lyrics: {}".format(LYRIC_FILE))
    # print("Passphrases: {}".format(PASS_FILE))


# if __name__ == '__main__':
#     main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk
# bopscrk - auxiliar functions module

import os, datetime


def clear():
    """Clear the screen. Works on Windows and Linux."""
    os.system(['clear', 'cls'][os.name == 'nt'])

def is_empty(variable):
    """
    Check if a variable (in a type convertible to string) is empty. Returns True or False
    """
    empty = False
    if len(str(variable)) == 0:
        empty = True
    return empty

def is_valid_date(date_str):
    """
    Check if a string corresponds to a valid date.
    :param date_str: date to check
    :return: True or False
    """
    try:
        datetime.datetime.strptime(date_str, '%d/%m/%Y')
        return True
    except ValueError:
        return False

def append_wordlist_to_file(filepath, wordlist):
    """
    Save wordlist into filepath provided (creates it if not exists, add words to the end if exists).
    :param filepath: path to file
    :param wordlist: list of words to save
    :return: True or False
    """
    try:
        with open(filepath, 'a') as f:
            for word in wordlist:
                f.write(word + '\n')
        return True
    except:
        return False


def remove_duplicates_from_file(infile_path, outfile_path="temp.000000000.bopscrk"):
    lines_seen = set()  # holds lines already seen
    outfile = open(outfile_path, "w")
    infile = open(infile_path, "r")
    for line in infile:
        if line not in lines_seen:  # not a duplicate
            outfile.write(line)
            lines_seen.add(line)
    outfile.close()
    infile.close()
    os.remove(infile_path)
    os.rename(outfile_path, infile_path)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk
# bopscrk - combinators functions module

import itertools

#from tqdm import tqdm
from alive_progress import alive_bar

from . import Config
from .excluders import remove_duplicates


def add_common_separators(wordlist):
    words = wordlist[:]
    new_wordlist = []
    for word in words:
        for separator in Config.SEPARATORS_CHARSET:
            new_wordlist.append(word + separator)
            new_wordlist.append(separator + word)

    base_wordlist_with_seps = new_wordlist[:]

    #with tqdm(total=len(words)) as progressbar:
    with alive_bar(total=len(words),bar='bubbles',unknown='bubbles',spinner='bubbles',receipt=False) as progressbar:
        for word in words:
            for wordseparated in base_wordlist_with_seps:
                if word not in wordseparated:
                    new_wordlist.append(wordseparated + word)
                    new_wordlist.append(word + wordseparated)
            #progressbar.update()
            progressbar()

    return remove_duplicates(new_wordlist)


def combinator(wordlist, nWords):
    new_wordlist = wordlist[:]  # I need copy to use itertools properly
    wlist_combined = itertools.permutations(new_wordlist, nWords)
    #list_combined_length = (sum(1 for _ in wlist_combined))
    wlist_combined = [''.join(i) for i in wlist_combined]

    #with tqdm(total=len(wlist_combined)) as progressbar:
    with alive_bar(total=len(wlist_combined), bar='bubbles', unknown='bubbles', spinner='bubbles',receipt=False) as progressbar:
        for combination in wlist_combined:
            #progressbar.set_description("Processing %s" % combination)
            word = ''
            for i in combination:
                word += i
            if word not in new_wordlist: new_wordlist.append(word)
            #progressbar.update()
            progressbar()

    return remove_duplicates(new_wordlist)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# https://github.com/r3nt0n/bopscrk

# packages with python3 setup.py -v sdist

from setuptools import setup, find_packages
from bopscrk.bopscrk import __version__, desc

# Read project description
with open('README.md', 'r') as f:
    long_desc = f.read()

setup(
    name='bopscrk',
    version=__version__,
    url='https://github.com/r3nt0n/bopscrk',
    author='r3nt0n',
    author_email='r3nt0n@protonmail.com',
    license='GNU General Public License v3.0',
    description=desc,
    long_description=long_desc,
    long_description_content_type="text/markdown",
    include_package_data=True,
    package_data={
        # If any package contains *.cfg files, include them
        '': ['*.cfg'],
    },
    #packages=['modules',],
    #packages=find_packages(),
    packages=['bopscrk', 'bopscrk.modules', 'bopscrk.modules.lyricpass'],
    #scripts=['bopscrk/bopscrk.py'],
    install_requires=['requests', 'alive-progress'],
    entry_points = {
        'console_scripts':[
            #'bopscrk = bopscrk.modules.main:run'
            'bopscrk = bopscrk.bopscrk:start'
        ]
    }
)
